import { __awaiter } from "tslib";
import { Observable } from '@equinor/workspace-core';
export class FetchController {
    constructor(fetch) {
        this.setFetch = (fetch) => {
            this.fetch = fetch;
        };
        this.reset = () => {
            this.data = undefined;
            this.currAbort = undefined;
            this.dataPromise = undefined;
            this.isLoading = false;
            this.isFetching = false;
        };
        /** Returns true if it aborted an api call */
        this.abort = () => {
            if (this.currAbort) {
                this.currAbort();
                return true;
            }
            return false;
        };
        /** Function for fetching data */
        this.getDataAsync = () => __awaiter(this, void 0, void 0, function* () {
            //Data is already fetching, just return the promise
            if (this.isFetching) {
                return yield this.dataPromise;
            }
            const { abort, signal } = new AbortController();
            this.currAbort = abort;
            if (!this.data) {
                this.setIsLoading(true);
            }
            this.setIsFetching(true);
            try {
                const promise = this.fetch(signal);
                this.dataPromise = promise;
                this.setData(yield promise);
                //Throwing the error further up the chain as it is not to be handled here, states still needs to be reset
                // eslint-disable-next-line no-useless-catch
            }
            catch (e) {
                throw e;
            }
            finally {
                this.setIsFetching(false);
                this.setIsLoading(false);
                this.dataPromise = undefined;
                this.currAbort = undefined;
            }
            return this.data;
        });
        /** True if its fetching for the first time */
        this.isLoading = false;
        /** True if currently fetching data */
        this.isFetching = false;
        this.destroy = () => {
            for (const key in this) {
                this[key] = null;
                delete this[key];
            }
        };
        this.fetch = fetch;
        const fetching = new Observable(this.isFetching, (a, b) => a === b);
        this.onIsFetchingChanged = fetching.onchange;
        this.setIsFetching = fetching.setValue;
        fetching.onchange((val) => {
            this.isFetching = val;
        });
        const loading = new Observable(this.isLoading, (a, b) => a === b);
        this.setIsLoading = loading.setValue;
        this.onIsLoadingChanged = loading.onchange;
        loading.onchange((val) => {
            this.isLoading = val;
        });
        const dataObservable = new Observable(this.data);
        this.setData = dataObservable.setValue;
        this.onDataChanged = dataObservable.onchange;
        dataObservable.onchange((val) => {
            this.data = val;
        });
    }
}
